@@ -2,7 +2,6 @@
 
 import os
 import sys
-import copy
 import time
 import numpy
 import cobra
@@ -10,8 +9,6 @@ import bisect
 import pandas
 import warnings
 import symengine
-import concurrent.futures
-from concurrent.futures import ProcessPoolExecutor
 from random import seed
 from cobra.util import solver
 from scipy.stats import spearmanr
@@ -28,7 +25,6 @@ class riptideClass:
         self.maximization_coefficients = 'NULL'
         self.pruned = 'NULL'
         self.flux_samples = 'NULL'
-        self.flux_variability = 'NULL'
         self.fraction_of_optimum = 'NULL'
         self.metabolic_tasks = 'NULL'
         self.concordance = 'NULL'
@@ -41,7 +37,7 @@ class riptideClass:
 
 
 # Save the output of RIPTiDe in a newly created directory
-def save_output(riptide_obj='NULL', path='NULL', file_type='JSON'):
+def save_output(riptide_obj='NULL', path='NULL', file_type='JSON', silent=False):
     '''Writes RIPTiDe results to files in a new directory
     
     Parameters
@@ -49,7 +45,7 @@ def save_output(riptide_obj='NULL', path='NULL', file_type='JSON'):
 
     REQUIRED
     riptide_obj : RIPTiDe object
-        Class object created by riptide.contextualize()
+        Class object created by riptide.contextualize() or riptide.maxfit()
 
     OPTIONAL
     path : str
@@ -58,18 +54,23 @@ def save_output(riptide_obj='NULL', path='NULL', file_type='JSON'):
         Type of output file for RIPTiDe model
         Accepts either sbml or json
         Default is JSON
+    silent : bool
+        Silences std out 
+        Default is False
     '''
 
     if riptide_obj == 'NULL':
         raise ValueError('ERROR: Did not provide a RIPTiDe object')
     
     if path == 'NULL':
-        print('WARNING: Did not provide an output directory. Using default riptide_files in working directory')
+        if silent == False:
+            print('WARNING: Did not provide an output directory. Using default riptide_files in working directory')
         path = 'riptide_files'
     try:
         os.mkdir(path)
     except:
-        print('WARNING: Output path already exists, overwriting previous files')
+        if silent == False:
+            print('WARNING: Output path already exists, overwriting previous files')
         pass
 
     # Write model to file
@@ -94,9 +95,6 @@ def save_output(riptide_obj='NULL', path='NULL', file_type='JSON'):
     if isinstance(riptide_obj.flux_samples, str) == False:
         outFile = path + '/flux_samples.tsv'
         riptide_obj.flux_samples.to_csv(outFile, sep='\t')
-    if isinstance(riptide_obj.flux_variability, str) == False:
-        outFile = path + '/flux_variability.tsv'
-        riptide_obj.flux_variability.to_csv(outFile, sep='\t')
 
     # Write coefficient dictionaries to tsvs
     outFile = path + '/flux_minimization_coefficients.tsv'
@@ -159,10 +157,6 @@ def save_output(riptide_obj='NULL', path='NULL', file_type='JSON'):
             parameters.write('Pooled transcript based on GPR: Yes\n')
         else:
             parameters.write('Pooled transcript based on GPR: No\n')
-        if riptide_obj.additional_parameters['open_exchanges'] == True:
-            parameters.write('Exchange reactions switched open: Yes\n')
-        else:
-            parameters.write('Exchange reactions switched open: No\n')
         if riptide_obj.additional_parameters['silent'] == True:
             parameters.write('Run in silent mode: Yes\n')
         else:
@@ -186,7 +180,7 @@ def save_output(riptide_obj='NULL', path='NULL', file_type='JSON'):
 
 # Read in transcriptomic read abundances, default is tsv with no header 
 def read_transcription_file(file, header = False, sep = '\t', rarefy = False, level = 'default',
-    binning = False, quant_max = 0.9, quant_min = 0.5, step = 0.125, norm = True, factor = 1e6):
+    binning = False, quant_max = 0.9, quant_min = 0.5, step = 0.125, norm = True, factor = 1e6, silent=False):
     '''Generates dictionary of transcriptomic abundances from a file
     
     Parameters
@@ -228,6 +222,9 @@ def read_transcription_file(file, header = False, sep = '\t', rarefy = False, le
     factor : numeric
         Denominator for read normalization calculation
         Default is 1e6 (RPM)
+    silent : bool
+        Silences std out 
+        Default is False
     '''
 
     # Correct some possible user error
@@ -245,12 +242,14 @@ def read_transcription_file(file, header = False, sep = '\t', rarefy = False, le
         if header == True: header_line = transcription.readline()
 
         for line in transcription:
-            line = line.split(sep)
-            abundances = [float(x) for x in line[1:]]
-            abund_dict[str(line[0])] = abundances
-            total_transcript += float(numpy.median(abundances))
-            min_transcript += float(min(abundances))
-            if reps < len(abundances): reps = len(abundances)
+            line = line.strip().split(sep)
+            abundances = line[1:]
+            if len(abundances) > 0:
+                abundances = [float(x) for x in abundances]
+                abund_dict[str(line[0])] = abundances
+                total_transcript += float(numpy.median(abundances))
+                min_transcript += float(min(abundances))
+                if reps < len(abundances): reps = len(abundances)
 
         abund_dict['replicates'] = reps
 
@@ -281,7 +280,8 @@ def read_transcription_file(file, header = False, sep = '\t', rarefy = False, le
 
     # If user-defined, perform abundance binning by quantile
     if binning == True:
-        print('Performing transcript abundance binning by quantile...')
+        if silent == False:
+            print('Performing transcript abundance binning by quantile...')
         abund_dict = _assign_quantiles(abund_dict, quant_max, quant_min, step)
 
     return abund_dict
@@ -337,56 +337,71 @@ def _rarefy(abunds, n):
     return list(result)
 
 
-# Version of riptide.contextualize compatible with multiprocessing
-def _iter_riptide(frac, argDict):
-    
-    iter = single_contextualize(model=argDict['model'], transcriptome=argDict['transcriptome'], fraction=frac, 
+# Finds the best Rho value from a list of riptide objects
+def _find_best_fit(frac_range, argDict, prev_best=None):
+    increment = 100.0 / float(len(frac_range))
+    progress = 0.0
+
+    if prev_best == None:
+        if argDict['silent'] == False:
+            print('Analyzing context-specific subnetwork flux ranges...')
+    else:
+        best_fit = prev_best
+        if argDict['silent'] == False:
+            print('Testing local objective fractions to ' + str(best_fit.fraction_of_optimum) + '...')
+
+    if argDict['silent'] == False:
+        sys.stdout.write('\rProgress: 0%     ')
+
+    maxfit_report = {}
+    best_fit = contextualize(model=argDict['model'], transcriptome=argDict['transcriptome'], fraction=frac_range[0], 
                          silent=True, samples=argDict['samples'], 
                          minimum=argDict['minimum'], conservative=argDict['conservative'], objective=argDict['objective'], 
                          additive=argDict['additive'], important=argDict['important'], set_bounds=argDict['set_bounds'], 
                          tasks=argDict['tasks'], exclude=argDict['exclude'], gpr=argDict['gpr'], threshold=argDict['threshold'], 
-                         open_exchanges=argDict['open_exchanges'])
-    return iter
-    
+                         phase=argDict['phase'])
+    maxfit_report[best_fit.fraction_of_optimum] = best_fit.concordance
 
-# Finds the best Rho value from a list of riptide objects
-def _find_best_fit(frac_range, argDict):
-    increment = 100.0 / float(len(frac_range))
-    progress = 0.0
-    sys.stdout.write('\rProgress: 0%')
-    
-    maxfit_report = {}
-    with concurrent.futures.ProcessPoolExecutor(max_workers=argDict['cpus']) as executor:
+    progress += increment
+    progress = float("%.2f" % progress)
+    if argDict['silent'] == False:
+        sys.stdout.write('\rProgress: ' + str(progress) + '%       ')
+        sys.stdout.flush()
 
-        maxfit_jobs = [executor.submit(_iter_riptide, frac, argDict) for frac in frac_range]
-        first_iter = 1
-        for job in concurrent.futures.as_completed(maxfit_jobs):
-            result = job.result()
-            progress += increment
-            progress = float("%.3f" % progress)
-            sys.stdout.write('\rProgress: ' + str(progress) + '%')
+    for frac in frac_range[1:]:
+        current_fit = contextualize(model=argDict['model'], transcriptome=argDict['transcriptome'], fraction=frac, 
+                         silent=True, samples=argDict['samples'], 
+                         minimum=argDict['minimum'], conservative=argDict['conservative'], objective=argDict['objective'], 
+                         additive=argDict['additive'], important=argDict['important'], set_bounds=argDict['set_bounds'], 
+                         tasks=argDict['tasks'], exclude=argDict['exclude'], gpr=argDict['gpr'], threshold=argDict['threshold'], 
+                         phase=argDict['phase'])
+        maxfit_report[current_fit.fraction_of_optimum] = current_fit.concordance
+        if current_fit.concordance['r'] > best_fit.concordance['r']: best_fit = current_fit
+
+        progress += increment
+        if progress > 100.: progress = 100.
+        progress = float("%.2f" % progress)
+        if argDict['silent'] == False:
+            sys.stdout.write('\rProgress: ' + str(progress) + '%       ')
             sys.stdout.flush()
 
-            maxfit_report[result.fraction_of_optimum] = result.concordance
-
-            if first_iter == 1:
-                best_fit = result
-                first_iter = 0
-            elif result.concordance['r'] > best_fit.concordance['r']:
-                best_fit = result
-
-    sys.stdout.write('\rProgress: 100%      \n\n')
+    if argDict['silent'] == False:
+        sys.stdout.write('\rProgress: 100%         \n\n')
+        sys.stdout.flush()
+        
     best_fit.maxfit_report = maxfit_report
 
     return best_fit
 
 
 # Iteratively run RIPTiDe over a range of objective minimum fractions
-def contextualize(model, transcriptome = 'none', frac_min = 0.25, frac_max = 0.85, 
-    samples = 500, cpus = 'all', minimum = False, conservative = False, objective = True, additive = False, 
-    important = [], set_bounds = True, silent = False, tasks = [], exclude = [], gpr = False, threshold = 1e-6, open_exchanges = False):
+def maxfit(model, transcriptome = 'none', frac_min = 0.25, frac_max = 0.85, frac_step = 0.1, prune = True,
+    samples = 1000, cpus = 'all', minimum = False, conservative = False, objective = True, additive = False, 
+    important = [], set_bounds = True, silent = False, tasks = [], exclude = [], gpr = False, threshold = 1e-5):
 
-    '''Iterative RIPTiDe for a range of minimum objective fluxes, returns model with best fit to transcriptome
+    '''
+    Iterative RIPTiDe for a range of minimum objective fluxes, returns model with best correlation 
+    with flux sampling results to transcriptome abundances.
     
     Parameters
     ----------
@@ -398,7 +413,7 @@ def contextualize(model, transcriptome = 'none', frac_min = 0.25, frac_max = 0.8
         Dictionary of transcript abundances, output of read_transcription_file()
     
     OPTIONAL
-    cpus  : int
+    cpus : int
         CPUs number for parallelization
         Default is all available 
     frac_min : float
@@ -407,10 +422,16 @@ def contextualize(model, transcriptome = 'none', frac_min = 0.25, frac_max = 0.8
     frac_max : float
         Upper bound for range of minimal fractions to test
         Default is 0.85
+    frac_step : float
+        Starting interval size within fraction range
+        Default is 0.1
+    prune : bool
+        Perform pruning step
+        Default is True
     samples : int 
         Number of flux samples to collect
-        Default is 500
-    silent  : bool
+        Default is 1000
+    silent : bool
         Silences std out 
         Default is False
     minimum : float
@@ -443,19 +464,10 @@ def contextualize(model, transcriptome = 'none', frac_min = 0.25, frac_max = 0.8
         Default is False
     threshold : float
         Minimum flux a reaction must acheive in order to avoid pruning during flux sum minimization step
-        Default is 1e-6
-    open_exchanges : bool
-        Sets all exchange reactions bounds to (-1000., 1000)
-        Default is False
-    skip_fva : bool
-        Skip final flux variability analysis step
-        Default is False
+        Default is 1e-5
     '''
 
     iter_start = time.time()
-    print('\nInitializing model and integrating transcriptomic data...')
-    print('Pruning zero flux subnetworks...')
-    print('Analyzing context-specific flux distributions...\n')
     seed(937162211)
 
     if samples <= 100:
@@ -468,58 +480,60 @@ def contextualize(model, transcriptome = 'none', frac_min = 0.25, frac_max = 0.8
         raise ValueError('ERROR: Invalid number of threads provided')
 
     if frac_min < 0. or frac_min > 1.:
-        print('WARNING: Improper minimum fraction provided, setting to default value')
+        if silent == False:
+            print('WARNING: Improper minimum fraction provided, setting to default value')
         frac_min = 0.25
     elif frac_min > frac_max:
-        print('WARNING: Improper minimum fraction provided, setting to default value')
+        if silent == False:
+            print('WARNING: Improper minimum fraction provided, setting to default value')
         frac_min = 0.25
 
     if frac_max < 0. or frac_max > 1.:
         print('WARNING: Improper maximum fraction provided, setting to default value')
         frac_max = 0.85
     elif frac_max < frac_min:
-        print('WARNING: Improper maximum fraction provided, setting to default value')
+        if silent == False:
+            print('WARNING: Improper maximum fraction provided, setting to default value')
         frac_max = 0.85
 
-    frac_step = 0.05
     frac_range = [round(x, 3) for x in list(numpy.arange(frac_min, frac_max+frac_step, frac_step))]
     if len(frac_range) == 1:
-        print('WARNING: Only a single fraction is possible in the input bounds and fraction')
+        if silent == False:
+            print('WARNING: Only a single fraction is possible in the input bounds and fraction')
 
     if silent == False:
-        print('\nRunning max fit RIPTiDe for objective fraction range:', frac_min, 'to', frac_max, '...')
+        print('Running max fit RIPTiDe for objective fraction range:', frac_min, 'to', frac_max, '...')
 
-    argDict = {'model':model, 'transcriptome':transcriptome, 'silent':True, 'cpus':cpus,
+    argDict = {'model':model, 'transcriptome':transcriptome, 'silent':silent, 'cpus':cpus,
                'samples':samples, 'minimum':minimum, 
                'conservative':conservative, 'objective':objective, 'additive':additive, 
                'important':important, 'set_bounds':set_bounds, 'tasks':tasks, 'exclude':exclude, 
-               'gpr':gpr, 'threshold':threshold, 'open_exchanges':open_exchanges}
-
+               'gpr':gpr, 'threshold':threshold, 'phase':2}
+    
+    warnings.filterwarnings('ignore', category=UserWarning)
     top_fit = _find_best_fit(frac_range, argDict)
     all_maxfit = top_fit.maxfit_report
     
-    if silent == False:
-            print('Testing local objective fractions to ' + str(top_fit.fraction_of_optimum) + '...')
+    small_frac_range = []
     while frac_step >= 0.025:
         frac_step = round(frac_step / 2.0, 3)
-        small_frac_range = [round(top_fit.fraction_of_optimum - frac_step, 3), top_fit.fraction_of_optimum, round(top_fit.fraction_of_optimum + frac_step, 3)]
-        new_fit = _find_best_fit(small_frac_range, argDict)
-
-        if new_fit.concordance['r'] <= top_fit.concordance['r']:
-            break
-        else:
-            top_fit = new_fit
-            for frac in new_fit.maxfit_report.keys():
-                all_maxfit[frac] = new_fit.maxfit_report[frac]
+        small_frac_range += [round(top_fit.fraction_of_optimum - frac_step, 3), round(top_fit.fraction_of_optimum + frac_step, 3)]
+    
+    new_fit = _find_best_fit(list(set(small_frac_range)), argDict, top_fit)
+    warnings.filterwarnings('default', category=UserWarning)
+    if new_fit.concordance['r'] > top_fit.concordance['r']:
+        top_fit = new_fit
+        for frac in new_fit.maxfit_report.keys():
+             all_maxfit[frac] = new_fit.maxfit_report[frac]
 
     top_fit.maxfit_report = all_maxfit
     top_fit.fraction_bounds = [frac_min, frac_max]
     top_fit.transcriptome = transcriptome
     if silent == False:
-        print('\nContext-specific metabolism fit with', top_fit.fraction_of_optimum, 'of optimal objective flux')
+        print('Context-specific metabolism fit with', top_fit.fraction_of_optimum, 'of optimal objective flux')
 
     # Analyze changes introduced by RIPTiDe and return results
-    report_dict = _operation_report(iter_start, model, top_fit.model, top_fit.concordance, silent, mf=True)
+    report_dict = _operation_report(iter_start, model, top_fit.model, top_fit.concordance, silent, phase=1)
     top_fit.additional_parameters['operation'] = report_dict
     top_fit.maxfit = True
 
@@ -527,9 +541,9 @@ def contextualize(model, transcriptome = 'none', frac_min = 0.25, frac_max = 0.8
 
 
 # Create context-specific model based on transcript distribution
-def single_contextualize(model, transcriptome = 'none', samples = 500, silent = False, 
+def contextualize(model, transcriptome = 'none', samples = 1000, silent = False, prune = True,
     fraction = 0.8, minimum = False, conservative = False, objective = True, additive = False, important = [], direct = False,
-    set_bounds = True, tasks = [], exclude = [], gpr = False, threshold = 1e-6, open_exchanges = False, skip_fva = False):
+    set_bounds = True, tasks = [], exclude = [], gpr = False, threshold = 1e-5, phase=1):
 
     '''Reaction Inclusion by Parsimony and Transcriptomic Distribution or RIPTiDe
     
@@ -552,7 +566,7 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
         Minimum objective fraction used during single run setting
         Default is 0.8
 
-    * Most other arguments from iterative implementation are carried over
+    * Other arguments from iterative implementation are carried over
     '''
 
     start_time = time.time()
@@ -566,7 +580,6 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
     riptide_object.additional_parameters['objective'] = objective
     riptide_object.additional_parameters['additive'] = additive
     riptide_object.additional_parameters['set_bounds'] = set_bounds
-    riptide_object.additional_parameters['open_exchanges'] = open_exchanges
 
     # Correct some possible user error
     samples = int(samples)
@@ -586,8 +599,8 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
     elif minimum == False:
         minimum = None
     riptide_object.additional_parameters['minimum'] = minimum
-    solution = model.slim_optimize()
-    if model.slim_optimize() < 1e-6 or str(model.slim_optimize()) == 'nan':
+    solution = model.slim_optimize(error_value=0.)
+    if solution < 1e-6:
         raise ValueError('ERROR: Provided model objective cannot carry flux! Please correct')
     minimum_threshold = threshold
     if isinstance(tasks, list) == False: tasks = [tasks]
@@ -618,28 +631,21 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
     riptide_object.fraction_of_optimum = fraction
     riptide_object.transcriptome = transcriptome
     riptide_object.gpr_integration = gpr
+    riptide_object.metabolic_tasks = tasks
 
     # Check original model functionality
     # Partition reactions based on transcription percentile intervals, assign corresponding reaction coefficients
-    if silent == False: print('\nInitializing model and integrating transcriptomic data...')
-    riptide_model = copy.deepcopy(model)
-    riptide_model.id = str(riptide_model.id) + '_riptide'
-    riptide_object.metabolic_tasks = tasks
-
-    # Open exchange reactions
-    if open_exchanges == True:
-        for rxn in riptide_model.exchanges: rxn.bounds = (-1000., 1000.)
-
-    # Remove totally blocked reactions to speed up subsequent sections
-    rm_rxns = list(set(exclude).difference(set(tasks)))
-    if len(rm_rxns) > 0:
-        riptide_model = _prune_model(riptide_model, rm_rxns, conservative)
-
+    if phase == 1:
+        if silent == False: 
+            print('\nInitializing model and integrating transcriptomic data...')
+    
     # Define linear coefficients for both steps
-    rxn_transcriptome, gene_hits = _transcript_to_reactions(transcriptome, riptide_model, gpr, additive)
+    rxn_transcriptome, gene_hits = _transcript_to_reactions(transcriptome, model, gpr, additive)
     keep_rxns = set()
     all_min_coefficient_dict = {}
-    if silent == False: print('Pruning zero flux subnetworks...')
+    if silent == False:
+        if phase == 1:
+            print('Pruning zero flux subnetworks...')
     for x in range(0, transcriptome['replicates']):
         current_rxn_transcriptome = {}
         for index in rxn_transcriptome.keys():
@@ -647,7 +653,7 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
                 continue
             else:
                 current_rxn_transcriptome[index] = rxn_transcriptome[index][x]
-        min_coefficient_dict, max_coefficient_dict, important_type = _assign_coefficients(current_rxn_transcriptome, riptide_model, important, direct)
+        min_coefficient_dict, max_coefficient_dict, important_type = _assign_coefficients(current_rxn_transcriptome, model, important, direct)
         for x in min_coefficient_dict.keys():
             try:
                 all_min_coefficient_dict[x].append(min_coefficient_dict[x])
@@ -655,19 +661,28 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
                 all_min_coefficient_dict[x] = [min_coefficient_dict[x]]
         
         # Determine active network sections based on coefficients
-        active_rxns = _constrain_and_analyze_model(model=riptide_model, coefficient_dict=min_coefficient_dict, fraction=fraction, sampling_depth=0, 
-            objective=objective, tasks=tasks, minimum_threshold=minimum_threshold, skip_fva=skip_fva)
+        active_rxns = _constrain_and_analyze_model(model=model, coefficient_dict=min_coefficient_dict, fraction=fraction, sampling_depth=0, 
+            objective=objective, tasks=tasks, minimum_threshold=minimum_threshold)
         keep_rxns = keep_rxns.union(active_rxns)
 
     # Determine inactive reactions and prune model
-    rm_rxns = set([x.id for x in riptide_model.reactions]).difference(keep_rxns)
-    riptide_model = _prune_model(riptide_model, rm_rxns, conservative)
-    riptide_object.pruned = _record_pruned_elements(model, riptide_model)
+    rm_rxns = set(list(set(exclude).difference(set(tasks))))
+    rm_rxns = set([x.id for x in model.reactions]).difference(keep_rxns)
+    rm_rxns = rm_rxns.difference(set(tasks))
+    rm_rxns = rm_rxns.union(set(exclude))
+    if len(rm_rxns) > 0 and prune == True:
+        riptide_model = _prune_model(model, rm_rxns, conservative)
+        riptide_object.pruned = _record_pruned_elements(model, riptide_model)
+    else:
+        riptide_model = model.copy()
+
     riptide_object.minimization_coefficients = all_min_coefficient_dict
     riptide_object.percent_of_mapping = gene_hits
 
     # Find optimal solution space based on transcription and final constraints
-    if silent == False: print('Analyzing context-specific flux distributions...')
+    if silent == False:
+        if phase == 1:
+            print('Analyzing context-specific flux distributions...')
     median_rxn_transcriptome = {}
     for x in rxn_transcriptome.keys(): 
         if x == 'replicates':
@@ -675,18 +690,22 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
         else:
             median_rxn_transcriptome[x] = numpy.median(rxn_transcriptome[x])
     min_coefficient_dict, max_coefficient_dict, important_type = _assign_coefficients(median_rxn_transcriptome, riptide_model, important, direct)
-    flux_samples, fva_result, concordance = _constrain_and_analyze_model(model=riptide_model, coefficient_dict=max_coefficient_dict, fraction=fraction, 
-        sampling_depth=samples, objective=objective, tasks=tasks, minimum_threshold=minimum_threshold, skip_fva=skip_fva)
+    flux_samples, concordance = _constrain_and_analyze_model(model=riptide_model, coefficient_dict=max_coefficient_dict, fraction=fraction, 
+        sampling_depth=samples, objective=objective, tasks=tasks, minimum_threshold=minimum_threshold)
     riptide_object.maximization_coefficients = max_coefficient_dict
     riptide_object.flux_samples = flux_samples
-    riptide_object.flux_variability = fva_result
     riptide_object.concordance = concordance
 
     # Assign new reaction bounds
-    if set_bounds == True and skip_fva == False: 
+    if set_bounds == True: 
         for rxn in riptide_model.reactions:
-            current_fva = list(fva_result.loc[rxn.id])
-            rxn.bounds = (min(current_fva), max(current_fva))
+            current_dist = list(flux_samples[rxn.id])
+            current_lb = min(current_dist)
+            current_ub = max(current_dist)
+            if current_lb == current_ub:
+                current_lb = current_lb * 0.9
+                current_ub = current_ub * 1.1
+            rxn.bounds = (current_lb, current_ub)
     riptide_object.model = riptide_model
 
     # Report on included subset of user-defined important genes
@@ -701,7 +720,7 @@ def single_contextualize(model, transcriptome = 'none', samples = 500, silent =
         riptide_object.included_important = important
 
     # Analyze changes introduced by RIPTiDe and return results
-    report_dict = _operation_report(start_time, model, riptide_model, concordance, silent, mf=False)
+    report_dict = _operation_report(start_time, model, riptide_model, concordance, silent, phase)
     riptide_object.additional_parameters['operation'] = report_dict
 
     return riptide_object
@@ -851,7 +870,7 @@ def _integrate_tasks(model, tasks):
     # Iteratively set each as the objective and find new bounds
     for rxn in screened_tasks:
         model.objective = rxn
-        task_obj_val = model.slim_optimize()
+        task_obj_val = model.slim_optimize(error_value=0.)
         # Check sign of objective value, just in case
         if task_obj_val > 0.0:
             task_constraint = model.problem.Constraint(model.objective.expression, lb=task_obj_val*0.01, ub=task_obj_val)
@@ -896,13 +915,14 @@ def _integrate_important(model, important, coefficient_dict):
 
 
 # Determine those reactions that carry flux in a pFBA objective set to a threshold of maximum
-def _constrain_and_analyze_model(model, coefficient_dict, fraction, sampling_depth, objective, tasks, minimum_threshold, skip_fva):
-    
-    constrained_model = copy.deepcopy(model)
+def _constrain_and_analyze_model(model, coefficient_dict, fraction, sampling_depth, objective, tasks, minimum_threshold):
+
+    constrained_model = model.copy()
+    constrained_model.id = str(constrained_model.id) + '_riptide'
 
     # Set previous objective as a constraint, allow deviation
     if objective == True:
-        prev_obj_val = constrained_model.slim_optimize()
+        prev_obj_val = constrained_model.slim_optimize(error_value=0.)
         prev_obj_constraint = constrained_model.problem.Constraint(constrained_model.objective.expression, lb=prev_obj_val*fraction, ub=prev_obj_val)
         constrained_model.add_cons_vars([prev_obj_constraint])
         constrained_model.solver.update()
@@ -933,8 +953,8 @@ def _constrain_and_analyze_model(model, coefficient_dict, fraction, sampling_dep
         # Explore solution space of constrained model with flux sampling, allow deviation
         constrained_model.objective = constrained_model.problem.Objective(pfba_expr, direction='max', sloppy=True)
         constrained_model.solver.update()
-        flux_sum_obj_val = constrained_model.slim_optimize()
-        flux_sum_constraint = constrained_model.problem.Constraint(pfba_expr, lb=flux_sum_obj_val*fraction, ub=flux_sum_obj_val)
+        flux_sum_obj_val = constrained_model.slim_optimize(error_value=0.)
+        flux_sum_constraint = constrained_model.problem.Constraint(pfba_expr, lb=flux_sum_obj_val*0.01, ub=flux_sum_obj_val)
         constrained_model.add_cons_vars([flux_sum_constraint])
         constrained_model.solver.update()
             
@@ -947,13 +967,8 @@ def _constrain_and_analyze_model(model, coefficient_dict, fraction, sampling_dep
         else:
             flux_samples = 'Not performed'
             concordance = 'Not performed'
-        
-        if skip_fva == False:
-            fva = flux_variability_analysis(constrained_model, fraction_of_optimum=fraction)
-        else:
-            fva = 'Not performed'
 
-        return flux_samples, fva, concordance
+    return flux_samples, concordance
 
 
 # Find level of concordance between contextualized flux and assigned coefficients
@@ -978,7 +993,9 @@ def _calc_concordance(flux_samples, coefficient_dict):
 
 
 # Prune model based on blocked reactions from minimization as well as user-defined reactions
-def _prune_model(new_model, rm_rxns, conserve):
+def _prune_model(model, rm_rxns, conserve):
+    
+    new_model = model.copy()
     
     # Parse elements highlighted for pruning based on GPRs
     if conserve == True:
@@ -1054,35 +1071,35 @@ def _complete_orphan_prune(model):
 
 
 # Reports how long RIPTiDe took to run
-def _operation_report(start_time, model, riptide, concordance, silent, mf):
+def _operation_report(start_time, model, riptide, concordance, silent, phase):
     report_dict = {}
 
     # Pruning
     perc_removal = 100.0 - ((float(len(riptide.reactions)) / float(len(model.reactions))) * 100.0)
     perc_removal = round(perc_removal, 2)
     if silent == False:
-        print('\nReactions pruned to ' + str(len(riptide.reactions)) + ' from ' + str(len(model.reactions)) + ' (' + str(perc_removal) + '% change)')
+        if phase == 1:
+            print('\nReactions pruned to ' + str(len(riptide.reactions)) + ' from ' + str(len(model.reactions)) + ' (' + str(perc_removal) + '% change)')
     report_dict['reactions'] = perc_removal
     perc_removal = 100.0 - ((float(len(riptide.metabolites)) / float(len(model.metabolites))) * 100.0)
     perc_removal = round(perc_removal, 2)
     if silent == False:
-        print('Metabolites pruned to ' + str(len(riptide.metabolites)) + ' from ' + str(len(model.metabolites)) + ' (' + str(perc_removal) + '% change)')
+        if phase == 1:
+            print('Metabolites pruned to ' + str(len(riptide.metabolites)) + ' from ' + str(len(model.metabolites)) + ' (' + str(perc_removal) + '% change)')
     report_dict['metabolites'] = perc_removal
 
     # Flux through objective
     model_check = 'works'
     # Check that prune model can still achieve flux through the objective (just in case)
     try:
-        if riptide.slim_optimize() < 1e-6 or str(riptide.slim_optimize()) == 'nan':
-            if silent == False:
-                print('WARNING: Contextualized model objective can no longer carry flux')
+        if riptide.slim_optimize(error_value=0.) < 1e-6:
             model_check = 'broken'
     except:
         pass
 
     if model_check == 'works':
-        new_ov = round(riptide.slim_optimize(), 2)
-        old_ov = round(model.slim_optimize(), 2)
+        new_ov = round(riptide.slim_optimize(error_value=0.), 2)
+        old_ov = round(model.slim_optimize(error_value=0.), 2)
         perc_shift = 100.0 - ((float(new_ov) / float(old_ov)) * 100.0)
         report_dict['obj_change'] = round(perc_shift, 2)
         if perc_shift == 0.0:
@@ -1090,11 +1107,13 @@ def _operation_report(start_time, model, riptide, concordance, silent, mf):
         elif perc_shift > 0.0:
             perc_shift = round(abs(perc_shift), 2)
             if silent == False:
-                print('Flux through the objective DECREASED to ~' + str(new_ov) + ' from ~' + str(old_ov) + ' (' + str(perc_shift) + '% change)')
+                if phase == 1:
+                    print('Flux through the objective DECREASED to ~' + str(new_ov) + ' from ~' + str(old_ov) + ' (' + str(perc_shift) + '% change)')
         elif perc_shift < 0.0:
             perc_shift = round(abs(perc_shift), 2)
             if silent == False:
-                print('Flux through the objective INCREASED to ~' + str(new_ov) + ' from ' + str(old_ov) + ' (' + str(perc_shift) + '% change)')
+                if phase == 1:
+                    print('Flux through the objective INCREASED to ~' + str(new_ov) + ' from ' + str(old_ov) + ' (' + str(perc_shift) + '% change)')
     else:
         report_dict['obj_change'] = 'fails'
 
@@ -1108,17 +1127,20 @@ def _operation_report(start_time, model, riptide, concordance, silent, mf):
             else:
                 p_val = 'p=' + str(p_val) + ' *'
             if silent == False:
-                print('Context-specific metabolism correlates with transcriptome (' + rho + ', ' + p_val + ')')
+                if phase == 1:
+                    print('Context-specific metabolism correlates with transcriptome (' + rho + ', ' + p_val + ')')
         else:
             if silent == False:
-                print('Context-specific metabolism does not correlate with transcriptome (' + rho + ', n.s.)')
+                if phase == 1:
+                    print('Context-specific metabolism does not correlate with transcriptome (' + rho + ', n.s.)')
 
     # Run time
     raw_seconds = int(round(time.time() - start_time))
     report_dict['run_time'] = raw_seconds
     if silent == False:
         if raw_seconds < 60:
-            print('\nRIPTiDe completed in ' + str(raw_seconds) + ' seconds\n')
+            if phase == 1:
+                print('\nRIPTiDe completed in ' + str(raw_seconds) + ' seconds\n')
         else:
             minutes, seconds = divmod(raw_seconds, 60)
             mins = 'minute'
@@ -1128,11 +1150,9 @@ def _operation_report(start_time, model, riptide, concordance, silent, mf):
             if seconds > 1:
                 secs = 'seconds'
             
-            if mf == False:
+            if phase == 1:
                 print('\nRIPTiDe completed in,', str(minutes), mins, 'and', str(int(seconds)), secs, '\n')
-            else:
-                print('\nMaxfit RIPTiDe completed in,', str(minutes), mins, 'and', str(int(seconds)), secs, '\n')
-
+            
     return report_dict
 
 #-----------------------------------------------------------------#
@@ -1142,7 +1162,7 @@ def _operation_report(start_time, model, riptide, concordance, silent, mf):
 # bioRxiv 652917; doi: https://doi.org/10.1101/652917 
 
 def _gapsplit(model, depth):
-    fva = flux_variability_analysis(model, model.reactions, fraction_of_optimum=0.001)
+    fva = flux_variability_analysis(model, list(model.reactions), fraction_of_optimum=0.001, processes=1)
 
     # only split reactions with feasible range >= min_range
     idxs = (fva.maximum - fva.minimum >= 1e-5).to_numpy().nonzero()[0]
